@@ -52,6 +52,7 @@ struct adhoc_state {
 	struct l_queue *sta_states;
 	uint32_t sta_watch_id;
 	uint32_t netdev_watch_id;
+	unsigned int mlme_watch;
 	struct l_dbus_message *pending;
 	uint32_t ciphers;
 	uint32_t group_cipher;
@@ -137,6 +138,7 @@ static void adhoc_reset(struct adhoc_state *adhoc)
 	adhoc->ssid = NULL;
 
 	netdev_station_watch_remove(adhoc->netdev, adhoc->sta_watch_id);
+	adhoc->sta_watch_id = 0;
 
 	l_queue_destroy(adhoc->sta_states, adhoc_sta_free);
 	adhoc->sta_states = NULL;
@@ -145,6 +147,9 @@ static void adhoc_reset(struct adhoc_state *adhoc)
 
 	l_dbus_property_changed(dbus_get_bus(), netdev_get_path(adhoc->netdev),
 						IWD_ADHOC_INTERFACE, "Started");
+
+	if (adhoc->mlme_watch)
+		l_genl_family_unregister(adhoc->nl80211, adhoc->mlme_watch);
 }
 
 static void adhoc_set_rsn_info(struct adhoc_state *adhoc,
@@ -179,11 +184,17 @@ static void adhoc_handshake_event(struct handshake_state *hs,
 {
 	struct sta_state *sta = user_data;
 	struct adhoc_state *adhoc = sta->adhoc;
+	va_list args;
 
 	switch (event) {
 	case HANDSHAKE_EVENT_FAILED:
-		l_error("handshake failed with STA "MAC, MAC_STR(sta->addr));
+		va_start(args, user_data);
 
+		l_error("handshake failed with STA "MAC" (%d)",
+						MAC_STR(sta->addr),
+						va_arg(args, int));
+
+		va_end(args);
 		/*
 		 * eapol frees the state machines upon handshake failure. Since
 		 * this is only a failure on one of the handshakes we need to
@@ -302,6 +313,7 @@ static void adhoc_start_rsna(struct sta_state *sta, const uint8_t *gtk_rsc)
 	eapol_register(sta->sm_a);
 
 	eapol_start(sta->sm);
+	eapol_start(sta->sm_a);
 
 	return;
 
@@ -482,16 +494,45 @@ static void adhoc_join_cb(struct netdev *netdev, int result, void *user_data)
 		return;
 	}
 
+	l_rtnl_set_linkmode_and_operstate(iwd_get_rtnl(),
+					netdev_get_ifindex(adhoc->netdev),
+					IF_LINK_MODE_DEFAULT, IF_OPER_UP,
+					NULL, NULL, NULL);
+
 	adhoc->sta_watch_id = netdev_station_watch_add(netdev,
 			adhoc_station_changed_cb, adhoc);
 
 	reply = l_dbus_message_new_method_return(adhoc->pending);
 	dbus_pending_reply(&adhoc->pending, reply);
+}
 
-	adhoc->started = true;
+static void adhoc_mlme_notify(struct l_genl_msg *msg, void *user_data)
+{
+	struct adhoc_state *adhoc = user_data;
+	uint32_t ifindex;
 
-	l_dbus_property_changed(dbus_get_bus(), netdev_get_path(adhoc->netdev),
+	if (nl80211_parse_attrs(msg, NL80211_ATTR_IFINDEX, &ifindex,
+				NL80211_ATTR_UNSPEC) < 0 ||
+			ifindex != netdev_get_ifindex(adhoc->netdev))
+		return;
+
+	switch (l_genl_msg_get_command(msg)) {
+	case NL80211_CMD_JOIN_IBSS:
+		/*
+		 * if watch is set the join_ibss_cb has come back. This event
+		 * will come in for each new STA joining the IBSS so we only
+		 * want to set it once
+		 */
+		if (adhoc->sta_watch_id && !adhoc->started) {
+			adhoc->started = true;
+
+			l_dbus_property_changed(dbus_get_bus(),
+						netdev_get_path(adhoc->netdev),
 						IWD_ADHOC_INTERFACE, "Started");
+		}
+
+		break;
+	}
 }
 
 static struct l_dbus_message *adhoc_dbus_start(struct l_dbus *dbus,
@@ -532,6 +573,11 @@ static struct l_dbus_message *adhoc_dbus_start(struct l_dbus *dbus,
 			adhoc))
 		return dbus_error_invalid_args(message);
 
+	adhoc->mlme_watch = l_genl_family_register(adhoc->nl80211, "mlme",
+						adhoc_mlme_notify, adhoc, NULL);
+	if (!adhoc->mlme_watch)
+		return dbus_error_failed(message);
+
 	return NULL;
 }
 
@@ -567,6 +613,11 @@ static struct l_dbus_message *adhoc_dbus_start_open(struct l_dbus *dbus,
 			adhoc))
 		return dbus_error_invalid_args(message);
 
+	adhoc->mlme_watch = l_genl_family_register(adhoc->nl80211, "mlme",
+						adhoc_mlme_notify, adhoc, NULL);
+	if (!adhoc->mlme_watch)
+		return dbus_error_failed(message);
+
 	return NULL;
 }
 
@@ -603,6 +654,11 @@ static struct l_dbus_message *adhoc_dbus_stop(struct l_dbus *dbus,
 	if (netdev_leave_adhoc(adhoc->netdev, adhoc_leave_cb, adhoc))
 		return dbus_error_failed(message);
 
+	l_rtnl_set_linkmode_and_operstate(iwd_get_rtnl(),
+					netdev_get_ifindex(adhoc->netdev),
+					IF_LINK_MODE_DORMANT, IF_OPER_DOWN,
+					NULL, NULL, NULL);
+
 	adhoc->pending = l_dbus_message_ref(message);
 
 	return NULL;
