@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2018  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,8 @@
 #include <ell/ell.h>
 
 #include "src/missing.h"
+#include "src/module.h"
+#include "src/util.h"
 #include "src/eap.h"
 #include "src/eap-private.h"
 #include "src/iwd.h"
@@ -55,11 +57,15 @@ struct eap_state {
 
 	struct eap_method *method;
 	char *identity;
+	char *identity_setting;
+	bool authenticator;
 
 	int last_id;
 	void *method_state;
 	bool method_success;
 	struct l_timeout *complete_timeout;
+	unsigned int identity_req_count;
+	unsigned int method_req_count;
 
 	bool discard_success_and_failure:1;
 };
@@ -121,11 +127,31 @@ static void eap_free_common(struct eap_state *eap)
 	if (eap->identity) {
 		l_free(eap->identity);
 		eap->identity = NULL;
+		l_free(eap->identity_setting);
+		eap->identity_setting = NULL;
 	}
 }
 
 void eap_free(struct eap_state *eap)
 {
+	/*
+	 * In supplicant mode we get here whether the authenticator has
+	 * sent an EAP-Failure or interrupted the connection at a higher
+	 * layer, or even when the negotiation stalled and we interrupted
+	 * it, so use this code path to check if it is the Identity value
+	 * that the authenticator may have a problem with.  We don't know
+	 * whether we're the phase 1 or phase 2 EAP state machine
+	 * (eap->set_key_material is a hint but doesn't work with EAP-PEAP
+	 * phase2) so print the setting name.
+	 */
+	if (!eap->authenticator && !eap->method_success) {
+		if (eap->identity_req_count && !eap->method_req_count)
+			l_info("EAP negotiation stopped after the Identity "
+				"exchange, this can happen when the %s value "
+				"is not what the authenticator expects",
+				eap->identity_setting ?: "identity");
+	}
+
 	eap_free_common(eap);
 	l_timeout_remove(eap->complete_timeout);
 
@@ -148,29 +174,19 @@ const char *eap_get_identity(struct eap_state *eap)
 	return eap->identity;
 }
 
-/**
- * eap_send_response:
- * @eap: EAP state
- * @type: Type of response being sent
- * @buf: Buffer to send
- * @len: Size of the buffer
- *
- * Sends out a response to a received request.  This method first fills the
- * EAP header into the buffer based on the EAP type response being sent.
- *
- * If the response type is EAP_TYPE_EXPANDED, then the Vendor-Id and
- * Vendor-Type fields are filled in based on contents of the eap_method
- * associated with @eap.
- *
- * The buffer passed in MUST be at least 12 bytes long if @type is
- * EAP_TYPE_EXPANDED and at least 5 bytes for other cases.
- **/
-void eap_send_response(struct eap_state *eap, enum eap_type type,
-						uint8_t *buf, size_t len)
+static void eap_send_packet(struct eap_state *eap, enum eap_code code,
+				uint8_t id, uint8_t *buf, size_t len)
 {
-	buf[0] = EAP_CODE_RESPONSE;
-	buf[1] = eap->last_id;
+	buf[0] = code;
+	buf[1] = id;
 	l_put_be16(len, &buf[2]);
+
+	eap->tx_packet(buf, len, eap->user_data);
+}
+
+static void eap_send_response(struct eap_state *eap, enum eap_type type,
+				uint8_t *buf, size_t len)
+{
 	buf[4] = type;
 
 	if (type == EAP_TYPE_EXPANDED) {
@@ -178,7 +194,41 @@ void eap_send_response(struct eap_state *eap, enum eap_type type,
 		l_put_be32(eap->method->vendor_type, buf + 8);
 	}
 
-	eap->tx_packet(buf, len, eap->user_data);
+	eap_send_packet(eap, EAP_CODE_RESPONSE, eap->last_id, buf, len);
+}
+
+/**
+ * eap_method_respond:
+ * @eap: EAP state
+ * @buf: Buffer to send
+ * @len: Size of the buffer
+ *
+ * Sends out a response to a received request.  This method first fills
+ * the EAP header in the buffer based on the method's EAP type being
+ * sent.
+ *
+ * If the method uses an expanded type , then the Vendor-Id and
+ * Vendor-Type fields are filled in automatically.
+ *
+ * The buffer passed in MUST be at least 12 bytes long if method uses
+ * an expanded type and at least 5 bytes for other cases.
+ **/
+void eap_method_respond(struct eap_state *eap, uint8_t *buf, size_t len)
+{
+	eap_send_response(eap, eap->method->request_type, buf, len);
+}
+
+void eap_method_new_request(struct eap_state *eap, uint8_t *buf, size_t len)
+{
+	buf[4] = eap->method->request_type;
+
+	if (eap->method->request_type == EAP_TYPE_EXPANDED) {
+		memcpy(buf + 5, eap->method->vendor_id, 3);
+		l_put_be32(eap->method->vendor_type, buf + 8);
+	}
+
+	/* TODO: Save copy for retransmissions, set timer, increment counter */
+	eap_send_packet(eap, EAP_CODE_REQUEST, ++eap->last_id, buf, len);
 }
 
 static void eap_complete_timeout(struct l_timeout *timeout, void *user_data)
@@ -213,6 +263,25 @@ static void eap_send_identity_response(struct eap_state *eap, char *identity)
 	eap_send_response(eap, EAP_TYPE_IDENTITY, buf, len + 5);
 }
 
+/* To be used in authenticator mode to trigger our first request */
+void eap_start(struct eap_state *eap)
+{
+	uint8_t buf[5];
+
+	L_WARN_ON(!eap->method || !eap->authenticator);
+
+	/*
+	 * Until we've received the Identity response we can resend the
+	 * Identity request with a constant ID on EAPoL-Start.
+	 */
+	if (eap->identity)
+		return;
+
+	eap->last_id = 1;
+	buf[4] = EAP_TYPE_IDENTITY;
+	eap_send_packet(eap, EAP_CODE_REQUEST, eap->last_id, buf, 5);
+}
+
 void __eap_handle_request(struct eap_state *eap, uint16_t id,
 				const uint8_t *pkt, size_t len)
 {
@@ -240,6 +309,8 @@ void __eap_handle_request(struct eap_state *eap, uint16_t id,
 		void (*op)(struct eap_state *eap,
 					const uint8_t *pkt, size_t len);
 
+		eap->method_req_count++;
+
 		if (type != eap->method->request_type) {
 			l_warn("EAP server tried method %i while client was "
 					"configured for method %i",
@@ -270,6 +341,8 @@ void __eap_handle_request(struct eap_state *eap, uint16_t id,
 
 	switch (type) {
 	case EAP_TYPE_IDENTITY:
+		eap->identity_req_count++;
+
 		if (len >= 2)
 			l_debug("Optional EAP server identity prompt: \"%.*s\"",
 					(int) len - 1, pkt + 1);
@@ -313,6 +386,162 @@ void __eap_handle_request(struct eap_state *eap, uint16_t id,
 	}
 }
 
+static const char *eap_type_to_str(enum eap_type type, uint32_t vendor_id,
+					uint32_t vendor_type)
+{
+	static char buf[100];
+
+	/*
+	 * Show vendor ID if type is expanded unless vendor_id and
+	 * vendor_type are 0 which means the peer is requesting an
+	 * expanded type through a Legacy Nak.  A Legacy Nak won't let
+	 * the peer specify the which expanded type it wants.
+	 *
+	 * TODO: Could also use names for known methods and/or vendors.
+	 */
+	if (type == EAP_TYPE_EXPANDED && vendor_id == 0 && vendor_type != 0)
+		type = vendor_type;
+
+	if (type != EAP_TYPE_EXPANDED || (vendor_id == 0 && vendor_type == 0)) {
+		snprintf(buf, sizeof(buf), "type(%i)", type);
+		return buf;
+	}
+
+	snprintf(buf, sizeof(buf), "vendor(%02x, %02x, %02x), type(%i)",
+			(vendor_id >> 16) & 255, (vendor_id >> 8) & 255,
+			vendor_id & 255, vendor_type);
+	return buf;
+}
+
+#define IS_EXPANDED_RESPONSE(id, t) \
+	(type == EAP_TYPE_EXPANDED && vendor_id == (id) && vendor_type == (t))
+
+static void eap_handle_response(struct eap_state *eap, const uint8_t *pkt,
+				size_t len)
+{
+	enum eap_type type;
+	uint32_t vendor_id;
+	uint32_t vendor_type;
+	enum eap_type our_type = eap->method->request_type;
+	uint32_t our_vendor_id = (eap->method->vendor_id[0] << 16) |
+				(eap->method->vendor_id[1] << 8) |
+				eap->method->vendor_id[2];
+	uint32_t our_vendor_type = eap->method->vendor_type;
+
+	bool response_is(enum eap_type wanted)
+	{
+		if (type == wanted)
+			return true;
+
+		return IS_EXPANDED_RESPONSE(0, wanted);
+	}
+
+	if (len < 1)
+		/* Invalid packets to be ignored */
+		return;
+
+	type = *pkt++;
+	len--;
+
+	if (type == EAP_TYPE_EXPANDED) {
+		if (len < 7)
+			return;
+
+		vendor_id = (pkt[0] << 16) | (pkt[1] << 8) | pkt[2];
+		vendor_type = l_get_be32(pkt + 3);
+		pkt += 7;
+		len -= 7;
+
+		if (vendor_id == 0 && vendor_type == EAP_TYPE_NAK &&
+				our_type != EAP_TYPE_EXPANDED)
+			/*
+			 * RFC3748 5.3.2: "[The Expanded Nak Type] MUST
+			 * be sent only in reply to a Request of Type 254.
+			 */
+			return;
+	}
+
+	if (response_is(EAP_TYPE_NAK)) {
+		l_debug("EAP peer not configured for method: %s",
+			eap_type_to_str(our_type, our_vendor_id,
+							our_vendor_type));
+
+		if ((type == EAP_TYPE_NAK && len == 1 && pkt[0] == 0) ||
+				(type != EAP_TYPE_NAK && len == 8 &&
+				 l_memeqzero(pkt, 8)))
+			l_debug("EAP peer proposed no alternative methods");
+		else if (type == EAP_TYPE_NAK)
+			while (len) {
+				l_debug("EAP peer proposed method: %s",
+					eap_type_to_str(*pkt++, 0, 0));
+				len--;
+			}
+		else
+			while (len >= 8) {
+				uint32_t v_id = (pkt[1] << 16) |
+						(pkt[2] << 8) |
+						pkt[3];
+
+				l_debug("EAP peer proposed method: %s",
+					eap_type_to_str(pkt[0], v_id,
+							l_get_be32(pkt + 4)));
+				pkt += 8;
+				len -= 8;
+			}
+
+		goto unsupported_method;
+	}
+
+	/*
+	 * If we don't have peer's identity yet it means we've only sent the
+	 * Identity Request so far so we expect an Identity Response and
+	 * nothing else.  Otherwise we only accept Response types matching
+	 * the method type and we forward responses directly to the method.
+	 */
+
+	if (!eap->identity) {
+		if (!response_is(EAP_TYPE_IDENTITY))
+			goto unsupported_method;
+
+		/*
+		 * RFC3748 Section 5.1: "The Identity Response field MUST NOT be
+		 * null terminated."
+		 * Only the Request can have data after a NUL char.
+		 */
+		if (len > 253 && memchr(pkt, '\0', len))
+			goto error;
+
+		eap->identity = l_strndup((char *) pkt, len);
+		if (eap->method->validate_identity &&
+				!eap->method->validate_identity(eap,
+								eap->identity))
+			goto error;
+
+		return;
+	}
+
+	/*
+	 * RFC3748 5.7: "An implementation that supports the Expanded
+	 * attribute MUST treat EAP Types that are less than 256 equivalently,
+	 * whether they appear as a single octet or as the 32-bit Vendor-Type
+	 * within an Expanded Type where Vendor-Id is 0."
+	 * (with the exception of the Nak)
+	 */
+	if (our_type != EAP_TYPE_EXPANDED) {
+		if (response_is(our_type))
+			goto handle_response;
+	} else if (IS_EXPANDED_RESPONSE(our_vendor_id, our_vendor_type))
+		goto handle_response;
+
+error:
+unsupported_method:
+	eap_method_error(eap);
+	return;
+
+handle_response:
+	eap->method->handle_response(eap, pkt, len);
+}
+
 void eap_rx_packet(struct eap_state *eap, const uint8_t *pkt, size_t len)
 {
 	uint8_t code, id;
@@ -328,11 +557,24 @@ void eap_rx_packet(struct eap_state *eap, const uint8_t *pkt, size_t len)
 
 	switch ((enum eap_code) code) {
 	case EAP_CODE_REQUEST:
+		if (eap->authenticator)
+			goto invalid;
+
 		__eap_handle_request(eap, id, pkt + 4, eap_len - 4);
 		return;
 
+	case EAP_CODE_RESPONSE:
+		if (!eap->authenticator || id != eap->last_id || !eap->method)
+			goto invalid;
+
+		eap_handle_response(eap, pkt + 4, eap_len - 4);
+		return;
+
 	case EAP_CODE_FAILURE:
 	case EAP_CODE_SUCCESS:
+		if (eap->authenticator)
+			goto invalid;
+
 		if (eap->discard_success_and_failure)
 			return;
 
@@ -353,8 +595,7 @@ void eap_rx_packet(struct eap_state *eap, const uint8_t *pkt, size_t len)
 			return;
 
 		if (eap_len != 4)
-			/* Invalid packets to be silently discarded */
-			return;
+			goto invalid;
 
 		if (code == EAP_CODE_SUCCESS && !eap->method_success)
 			/* "Canned" success packets to be discarded */
@@ -380,6 +621,7 @@ void eap_rx_packet(struct eap_state *eap, const uint8_t *pkt, size_t len)
 				EAP_RESULT_FAIL, eap->user_data);
 		return;
 
+	invalid:
 	default:
 		/* Invalid packets to be silently discarded */
 		return;
@@ -551,11 +793,17 @@ bool eap_load_settings(struct eap_state *eap, struct l_settings *settings,
 		if (!eap->method->load_settings(eap, settings, prefix))
 			goto err;
 
+	eap->authenticator = eap->method->handle_request == NULL;
+
+	if (eap->authenticator)
+		return true;
+
 	/* get identity from settings or from EAP method */
 	if (!eap->method->get_identity) {
 		snprintf(setting, sizeof(setting), "%sIdentity", prefix);
 		eap->identity = l_settings_get_string(settings,
 							"Security", setting);
+		eap->identity_setting = l_strdup(setting);
 	} else {
 		eap->identity = l_strdup(eap->method->get_identity(eap));
 	}
@@ -567,10 +815,17 @@ bool eap_load_settings(struct eap_state *eap, struct l_settings *settings,
 	 * octets. Support for an NAI length of 253 octets is RECOMMENDED.
 	 * ...
 	 * RADIUS is unable to support NAI lengths beyond 253 octets
+	 *
+	 * We also need to fail if the identity is too large for the set MTU
+	 * size minus 5 (header).
 	 */
-	if (eap->identity && strlen(eap->identity) > 253) {
-		l_error("Identity is too long");
-		goto err;
+	if (eap->identity) {
+		size_t id_len = strlen(eap->identity);
+
+		if (id_len > 253 || id_len > eap->mtu - 5) {
+			l_error("Identity is too long");
+			goto err;
+		}
 	}
 
 	return true;
@@ -631,6 +886,14 @@ bool eap_method_is_success(struct eap_state *eap)
 void eap_method_success(struct eap_state *eap)
 {
 	eap->method_success = true;
+
+	if (eap->authenticator) {
+		uint8_t buf[4];
+
+		/* ID not incremented for Success or Failure (RFC3748 4.2) */
+		eap_send_packet(eap, EAP_CODE_FAILURE, eap->last_id, buf, 4);
+		eap->complete(EAP_RESULT_SUCCESS, eap->user_data);
+	}
 }
 
 void eap_discard_success_and_failure(struct eap_state *eap, bool discard)
@@ -640,6 +903,13 @@ void eap_discard_success_and_failure(struct eap_state *eap, bool discard)
 
 void eap_method_error(struct eap_state *eap)
 {
+	if (eap->authenticator) {
+		uint8_t buf[4];
+
+		/* ID not incremented for Success or Failure (RFC3748 4.2) */
+		eap_send_packet(eap, EAP_CODE_FAILURE, eap->last_id, buf, 4);
+	}
+
 	/*
 	 * It looks like neither EAP nor EAP-TLS specify the error handling
 	 * behavior.
@@ -659,7 +929,7 @@ void eap_restore_last_id(struct eap_state *eap, uint8_t last_id)
 
 int eap_register_method(struct eap_method *method)
 {
-	if (!method->handle_request)
+	if (!method->handle_request && !method->handle_response)
 		return -EPERM;
 
 	l_queue_push_head(eap_methods, method);
@@ -679,7 +949,7 @@ int eap_unregister_method(struct eap_method *method)
 
 void __eap_set_config(struct l_settings *config)
 {
-	if (!l_settings_get_uint(config, "EAP", "mtu", &default_mtu))
+	if (!l_settings_get_uint(config, "EAP", "MTU", &default_mtu))
 		default_mtu = 1400; /* on WiFi the real MTU is around 2304 */
 }
 
